# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os

from socket import gethostname

import requests

from circus.plugins import CircusPlugin
from zmq.eventloop import ioloop


class ProcfileWatcher(CircusPlugin):
    name = "procfile_watcher"

    def handle_recv(self, data):
        pass


class StatusReporter(CircusPlugin):
    name = "status_reporter"

    def __init__(self, *args, **config):
        super(StatusReporter, self).__init__(*args, **config)
        self.loop_rate = int(config.get("loop_rate", 60))
        self.hostname = gethostname()
        self.period = ioloop.PeriodicCallback(self.report,
                                              self.loop_rate * 1000,
                                              self.loop)

    def handle_init(self):
        self.period.start()

    def handle_stop(self):
        self.period.stop()

    def handle_recv(self, data):
        pass

    def report(self):
        cmds = self.call("status")["statuses"]
        status = "started"
        for cmd_name, cmd_status in cmds.iteritems():
            if cmd_name.startswith("plugin:"):
                continue
            if cmd_status != "active":
                status = "error"
                break
        envs = os.environ
        url_data = {"host": envs.get("TSURU_HOST"),
                    "appname": envs.get("TSURU_APPNAME"),
                    "unitname": self.hostname}
        url = "{host}/apps/{appname}/units/{unitname}"
        token = envs.get("TSURU_APP_TOKEN")
        requests.post(url.format(**url_data), data={"status": status},
                      headers={"Authorization": "bearer " + token})



# Copyright 2014 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import tsuru_unit_agent.stream


# All implementation has moved to tsuru-unit-agent, this class is only defined
# here so that we don't break existing circus.ini files.
class Stream(tsuru_unit_agent.stream.Stream):
    pass



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from tornado.testing import gen_test
from circus.tests.support import TestCircus, async_poll_for
from circus.util import DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB
from circus.util import tornado_sleep
from tsuru.plugins.statsd import Stats, storages
from tsuru.plugins.stats.fake import FakeBackend
from tsuru.plugins.stats.statsd import StatsdBackend
from tsuru.plugins.stats.logstash import LogstashBackend
from mock import patch, Mock
import tornado

from time import time
import multiprocessing
import functools
import os


def run_plugin(cls, config, plugin_info_callback=None, duration=300,
               endpoint=DEFAULT_ENDPOINT_DEALER, pubsub_endpoint=DEFAULT_ENDPOINT_SUB):
    check_delay = 1
    ssh_server = None

    plugin = cls(endpoint, pubsub_endpoint, check_delay, ssh_server, **config)

    if hasattr(plugin, 'storage'):
        plugin.storage.stop()

    deadline = time() + (duration / 1000.)
    plugin.loop.add_timeout(deadline, plugin.stop)

    plugin.start()
    try:
        if plugin_info_callback:
            plugin_info_callback(plugin)
    finally:
        plugin.stop()

    return plugin.storage


@tornado.gen.coroutine
def async_run_plugin(cls, config, plugin_info_callback, duration=300,
                     endpoint=DEFAULT_ENDPOINT_DEALER, pubsub_endpoint=DEFAULT_ENDPOINT_SUB):
    queue = multiprocessing.Queue()
    plugin_info_callback = functools.partial(plugin_info_callback, queue)
    circusctl_process = multiprocessing.Process(
        target=run_plugin,
        args=(cls, config, plugin_info_callback, duration, endpoint, pubsub_endpoint))
    circusctl_process.start()

    while queue.empty():
        yield tornado_sleep(.1)

    result = queue.get()
    raise tornado.gen.Return(result)


def get_gauges(queue, plugin):
    queue.put(plugin.storage.gauges)


class TestStats(TestCircus):

    @gen_test
    def test_stats(self):
        envs = {
            "TSURU_METRICS_BACKEND": "fake",
        }
        os.environ.update(envs)
        dummy_process = 'circus.tests.support.run_process'
        yield self.start_arbiter(dummy_process)
        async_poll_for(self.test_file, 'START')

        config = {'loop_rate': 0.2}
        stats_class = Stats
        stats_class.disk_usage = lambda x: 0
        stats_class.net_io = lambda x: (0, 0)
        stats_class.connections_established = lambda x: 0
        stats_class.connections = lambda x: []
        gauges = yield async_run_plugin(
            stats_class, config,
            plugin_info_callback=get_gauges,
            duration=1000,
            endpoint=self.arbiter.endpoint,
            pubsub_endpoint=self.arbiter.pubsub_endpoint
        )

        # we should have a bunch of stats events here
        self.assertTrue(len(gauges) >= 5)
        last_batch = sorted(name for name, value in gauges[-5:])
        wanted = ['test.cpu_sum', 'test.mem_max',
                  'test.mem_pct_max', 'test.mem_pct_sum',
                  'test.mem_sum']
        self.assertEqual(last_batch, wanted)

        yield self.stop_arbiter()

    @patch("psutil.disk_usage")
    def test_disk_usage(self, disk_usage_mock):
        stats = Stats("endpoint", "pubsub", 1.0, "ssh_server")
        stats.disk_usage()

        disk_usage_mock.assert_called_with("/")

    @patch("psutil.net_io_counters")
    def test_net_io(self, net_io_mock):
        stats = Stats("endpoint", "pubsub", 1.0, "ssh_server")
        stats.net_io()

        net_io_mock.assert_called_with()

    @patch("psutil.net_connections")
    def test_connections_established(self, conn_mock):
        conn = Mock(status="ESTABLISHED")
        conn_mock.return_value = [conn, conn, conn, conn, conn]

        stats = Stats("endpoint", "pubsub", 1.0, "ssh_server")
        established = stats.connections_established()

        self.assertEqual(5, established)

        conn_mock.assert_called_with("tcp")

    def test_get_storage(self):
        envs = {
            "TSURU_METRICS_BACKEND": "fake",
        }
        os.environ.update(envs)
        stats = Stats("endpoint", "pubsub", 1.0, "ssh_server")
        storage = stats.get_storage()
        self.assertIsInstance(storage, FakeBackend)

    def test_statsd_default_storage(self):
        del os.environ["TSURU_METRICS_BACKEND"]
        stats = Stats("endpoint", "pubsub", 1.0, "ssh_server")
        storage = stats.get_storage()
        self.assertIsInstance(storage, StatsdBackend)

    def test_logstash_registered(self):
        self.assertIn("logstash", storages)
        self.assertIsInstance(storages["logstash"](), LogstashBackend)

    def test_get_a_not_registered_backend(self):
        envs = {
            "TSURU_METRICS_BACKEND": "doesnotexist",
        }
        os.environ.update(envs)
        stats = Stats("endpoint", "pubsub", 1.0, "ssh_server")
        storage = stats.get_storage()
        # when a backend does not exists the default should be returned
        self.assertIsInstance(storage, StatsdBackend)



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase

from tsuru.plugins import ProcfileWatcher


class ProcfileWatcherTest(TestCase):
    def test_plugin(self):
        plugin = ProcfileWatcher("", "", 1)
        self.assertIsInstance(plugin, ProcfileWatcher)



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os
import socket


class StatsdBackend(object):

    def __init__(self):
        envs = os.environ

        app_name = envs.get("TSURU_APPNAME")
        host_name = socket.gethostname()
        # tsuru.app.host
        self.prefix = 'tsuru.{}.{}'.format(app_name, host_name)

        # initialize statsd
        from circus.plugins.statsd import StatsdClient
        host = envs.get("STATSD_HOST", 'localhost') or "localhost"
        port = envs.get("STATSD_PORT", '8125') or "8125"

        self.client = StatsdClient(
            host=host,
            port=int(port),
            prefix=self.prefix,
            sample_rate=1.0
        )

    def stop(self):
        self.client.stop()

    def gauge(self, key, value):
        self.client.gauge(key, value)

    def disk_usage(self, value):
        self.gauge("disk_usage", value)

    def net_sent(self, value):
        self.gauge("net.sent", value)

    def net_recv(self, value):
        self.gauge("net.recv", value)

    def net_connections(self, value):
        self.gauge("net.connections", value)

    def connections(self, value):
        pass

    def cpu_max(self, name, value):
        self.gauge("{}.cpu_max".format(name), value)

    def cpu_sum(self, name, value):
        self.gauge("{}.cpu_sum".format(name), value)

    def mem_max(self, name, value):
        self.gauge("{}.mem_max".format(name), value)

    def mem_sum(self, name, value):
        self.gauge("{}.mem_sum".format(name), value)

    def mem_pct_max(self, name, value):
        self.gauge("{}.mem_pct_max".format(name), value)

    def mem_pct_sum(self, name, value):
        self.gauge("{}.mem_pct_sum".format(name), value)



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os
import socket


class LogstashBackend(object):

    def __init__(self):
        envs = os.environ

        self.app_name = envs.get("TSURU_APPNAME")
        self.host_name = socket.gethostname()

        # initialize measure
        host = envs.get("LOGSTASH_HOST") or 'localhost'
        port = envs.get("LOGSTASH_PORT") or '1984'

        from measures import Measure
        self.client = Measure(envs.get("LOGSTASH_CLIENT", "tsuru"), (host, int(port)))

    def stop(self):
        pass

    def gauge(self, key, value):
        dimensions = {"app": self.app_name, "host": self.host_name, "value": float(value)}
        self.client.count(key, dimensions=dimensions)

    def disk_usage(self, value):
        self.gauge("disk_usage", value)

    def net_sent(self, value):
        self.gauge("net_sent", value)

    def net_recv(self, value):
        self.gauge("net_recv", value)

    def net_connections(self, value):
        self.gauge("net_connections", value)

    def connections(self, connection_list):
        for conn in connection_list:
            dimensions = {"app": self.app_name, "host": self.host_name, "connection": conn}
            self.client.count("connection", dimensions=dimensions)

    def cpu_max(self, name, value):
        self.gauge("cpu_max", value)

    def cpu_sum(self, name, value):
        self.gauge("cpu_sum", value)

    def mem_max(self, name, value):
        self.gauge("mem_max", value)

    def mem_sum(self, name, value):
        self.gauge("mem_sum", value)

    def mem_pct_max(self, name, value):
        self.gauge("mem_pct_max", value)

    def mem_pct_sum(self, name, value):
        self.gauge("mem_pct_sum", value)



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase
import os

from tsuru.plugins.stats.logstash import LogstashBackend
from mock import patch, Mock


class TestLogstash(TestCase):

    @patch("socket.gethostname")
    def setUp(self, gethostname_mock):
        gethostname_mock.return_value = "somehost"
        envs = {
            "TSURU_APPNAME": "appname",
        }
        os.environ.update(envs)
        self.logstash = LogstashBackend()
        self.logstash.gauge = Mock()

    @patch("measures.Measure")
    @patch("socket.gethostname")
    def test_logstash_environs(self, gethostname_mock, measure_mock):
        gethostname_mock.return_value = "somehost"
        envs = {
            "TSURU_APPNAME": "appname",
        }
        os.environ.update(envs)
        LogstashBackend()
        measure_mock.assert_called_with('tsuru', ('localhost', 1984))

        envs = {
            "TSURU_APPNAME": "appname",
            "LOGSTASH_HOST": "logstashhost",
            "LOGSTASH_PORT": "123",
            "LOGSTASH_CLIENT": "clientstash",
        }
        os.environ.update(envs)
        LogstashBackend()
        measure_mock.assert_called_with('clientstash', ('logstashhost', 123))

    def test_init(self):
        self.assertEqual(self.logstash.app_name, "appname")
        self.assertEqual(self.logstash.host_name, "somehost")

    def test_mem_max(self):
        self.logstash.mem_max("name", 123)
        self.logstash.gauge.assert_called_with('mem_max', 123)

    def test_cpu_max(self):
        self.logstash.cpu_max("name", 123)
        self.logstash.gauge.assert_called_with('cpu_max', 123)

    def test_mem_sum(self):
        self.logstash.mem_sum("name", 123)
        self.logstash.gauge.assert_called_with('mem_sum', 123)

    def test_cpu_sum(self):
        self.logstash.cpu_sum("name", 123)
        self.logstash.gauge.assert_called_with('cpu_sum', 123)

    def test_mem_pct_sum(self):
        self.logstash.mem_pct_sum("name", 123)
        self.logstash.gauge.assert_called_with('mem_pct_sum', 123)

    def test_mem_pct_max(self):
        self.logstash.mem_pct_max("name", 123)
        self.logstash.gauge.assert_called_with('mem_pct_max', 123)

    def test_disk_usage(self):
        self.logstash.disk_usage(123)
        self.logstash.gauge.assert_called_with('disk_usage', 123)

    def test_net_sent(self):
        self.logstash.net_sent(123)
        self.logstash.gauge.assert_called_with('net_sent', 123)

    def test_net_recv(self):
        self.logstash.net_recv(123)
        self.logstash.gauge.assert_called_with('net_recv', 123)

    def test_net_connections(self):
        self.logstash.net_connections(123)
        self.logstash.gauge.assert_called_with('net_connections', 123)

    @patch("socket.gethostname")
    def test_gauge(self, gethostname_mock):
        gethostname_mock.return_value = "somehost"
        envs = {
            "TSURU_APPNAME": "appname",
        }
        os.environ.update(envs)
        logstash = LogstashBackend()
        logstash.client = Mock()

        logstash.gauge("key", 1)

        dimensions = {'app': 'appname', 'host': 'somehost', 'value': 1.0}
        logstash.client.count.assert_called_with('key', dimensions=dimensions)



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import socket
import unittest
import mock
import os

from tsuru.plugins import StatusReporter


class StatusReporterTestCase(unittest.TestCase):

    def test_init(self):
        hostname = socket.gethostname()
        status_reporter = StatusReporter("", "", 1,
                                         loop_rate="180",
                                         apprc="/etc/apprc")
        self.assertEqual(180, status_reporter.loop_rate)
        self.assertEqual(hostname, status_reporter.hostname)

    def test_init_default_config_values(self):
        status_reporter = StatusReporter("", "", 1)
        self.assertEqual(60, status_reporter.loop_rate)

    def test_handle_init(self):
        status_reporter = StatusReporter("", "", 1)
        status_reporter.period.start = mock.Mock()
        status_reporter.handle_init()
        status_reporter.period.start.assert_called_once()

    def test_handle_stop(self):
        status_reporter = StatusReporter("", "", 1)
        status_reporter.period.stop = mock.Mock()
        status_reporter.handle_stop()
        status_reporter.period.stop.assert_called_once()

    @mock.patch("requests.post")
    @mock.patch("tsuru.plugins.gethostname")
    def test_report_ignores_plugins(self, gethostname, post):
        gethostname.return_value = "myhost"
        status_reporter = StatusReporter("", "", 1)
        envs = {
            "TSURU_HOST": "http://tsuru.io:8080",
            "TSURU_APP_TOKEN": "abc123",
            "TSURU_APPNAME": "something"
        }
        os.environ.update(envs)
        call = mock.Mock()
        call.return_value = {"statuses": {"plugin:tsuru-hooks": "stopped",
                                          "something": "active"}}
        status_reporter.call = call
        status_reporter.report()
        call.assert_called_once()
        url = "http://tsuru.io:8080/apps/something/units/myhost"
        post.assert_called_with(url, data={"status": "started"},
                                headers={"Authorization": "bearer abc123"})

    @mock.patch("requests.post")
    @mock.patch("tsuru.plugins.gethostname")
    def test_report_non_active_process(self, gethostname, post):
        gethostname.return_value = "myhost"
        status_reporter = StatusReporter("", "", 1)
        envs = {
            "TSURU_HOST": "http://tsuru.io:8080",
            "TSURU_APP_TOKEN": "abc123",
            "TSURU_APPNAME": "something"
        }
        os.environ.update(envs)
        call = mock.Mock()
        call.return_value = {"statuses": {"otherthing": "active",
                                          "something": "active",
                                          "wat": "stopped"}}
        status_reporter.call = call
        status_reporter.report()
        call.assert_called_once()
        url = "http://tsuru.io:8080/apps/something/units/myhost"
        post.assert_called_with(url, data={"status": "error"},
                                headers={"Authorization": "bearer abc123"})



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from zmq.eventloop import ioloop

from circus.plugins import CircusPlugin
from circus.util import human2bytes
from tsuru.plugins.stats.statsd import StatsdBackend
from tsuru.plugins.stats.logstash import LogstashBackend
from tsuru.plugins.stats.fake import FakeBackend

import psutil
import os


storages = {
    "fake": FakeBackend,
    "statsd": StatsdBackend,
    "logstash": LogstashBackend,
}


class StatsdEmitter(CircusPlugin):
    name = 'statsd'
    default_app_name = "app"

    def __init__(self, endpoint, pubsub_endpoint, check_delay, ssh_server,
                 **config):
        super(StatsdEmitter, self).__init__(endpoint, pubsub_endpoint,
                                            check_delay, ssh_server=ssh_server)

        self.storage = self.get_storage()

    def get_storage(self):
        default_backend = "statsd"
        storage = os.environ.get("TSURU_METRICS_BACKEND", default_backend)

        # when a backend does not exists the default should be returned
        if storage not in storages:
            storage = default_backend

        return storages[storage]()

    def handle_recv(self, data):
        pass

    def stop(self):
        self.storage.stop()
        super(StatsdEmitter, self).stop()


class BaseObserver(StatsdEmitter):

    def __init__(self, *args, **config):
        super(BaseObserver, self).__init__(*args, **config)
        self.loop_rate = float(config.get("loop_rate", 60))  # in seconds

    def handle_init(self):
        self.period = ioloop.PeriodicCallback(self.look_after,
                                              self.loop_rate * 1000, self.loop)
        self.period.start()

    def handle_stop(self):
        self.period.stop()

    def handle_recv(self, data):
        pass

    def look_after(self):
        raise NotImplementedError()


class Stats(BaseObserver):

    name = 'stats'

    def disk_usage(self):
        return psutil.disk_usage("/").used

    def net_io(self):
        io = psutil.net_io_counters()
        return io.bytes_sent, io.bytes_recv

    def connections_established(self):
        connections = psutil.net_connections("tcp")
        return sum([1 for conn in connections if conn.status == "ESTABLISHED"])

    def connections(self):
        result = psutil.net_connections("tcp")
        conns = []

        for conn in result:
            if conn.status == "ESTABLISHED" and conn.raddr[0] != "127.0.0.1":
                ip, port = conn.raddr

                if conn.laddr[1] == 8888:
                    ip = "127.0.0.1"
                    port = conn.laddr[1]

                conns.append("{}:{}".format(ip, port))

        return conns

    def look_after(self):
        self.storage.disk_usage(self.disk_usage())

        net_sent, net_recv = self.net_io()
        self.storage.net_sent(net_sent)
        self.storage.net_recv(net_recv)

        self.storage.net_connections(self.connections_established())
        self.storage.connections(self.connections())

        info = self.call("stats")
        if info["status"] == "error":
            return
        for name, stats in info['infos'].items():
            if name.startswith("plugin:"):
                # ignore plugins
                continue

            cpus = []
            mems = []
            mem_infos = []

            for sub_name, sub_info in stats.items():
                if isinstance(sub_info, dict):
                    cpu = [sub_info['cpu']]
                    mem = [sub_info['mem']]
                    inf = [human2bytes(sub_info['mem_info1'])]

                    for p in sub_info['children']:
                        cpu.append(p['cpu'])
                        mem.append(p['mem'])
                        inf.append(human2bytes(p['mem_info1']))

                    cpus.append(sum(cpu))
                    mems.append(sum(mem))
                    mem_infos.append(sum(inf))

                if not cpus:
                    # if there are only dead processes, we have an empty list
                    # and we can't measure it
                    continue

                self.storage.cpu_max(name, max(cpus))
                self.storage.cpu_sum(name, sum(cpus))
                self.storage.mem_pct_max(name, max(mems))
                self.storage.mem_pct_sum(name, sum(mems))
                self.storage.mem_max(name, max(mem_infos))
                self.storage.mem_sum(name, sum(mem_infos))



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


class FakeBackend(object):

    def __init__(self):
        self.gauges = []

    def stop(self):
        pass

    def gauge(self, key, value):
        self.gauges.append((key, value))

    def disk_usage(self, value):
        self.gauge("disk_usage", value)

    def net_sent(self, value):
        self.gauge("net.sent", value)

    def net_recv(self, value):
        self.gauge("net.recv", value)

    def net_connections(self, value):
        self.gauge("net.connections", value)

    def connections(self, conns):
        for conn in conns:
            self.gauge("connection", conn)

    def cpu_max(self, name, value):
        self.gauge("{}.cpu_max".format(name), value)

    def cpu_sum(self, name, value):
        self.gauge("{}.cpu_sum".format(name), value)

    def mem_max(self, name, value):
        self.gauge("{}.mem_max".format(name), value)

    def mem_sum(self, name, value):
        self.gauge("{}.mem_sum".format(name), value)

    def mem_pct_max(self, name, value):
        self.gauge("{}.mem_pct_max".format(name), value)

    def mem_pct_sum(self, name, value):
        self.gauge("{}.mem_pct_sum".format(name), value)



# Copyright 2015 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from unittest import TestCase
import os

from tsuru.plugins.stats.statsd import StatsdBackend
from mock import patch, Mock


class TestStatsd(TestCase):

    @patch("socket.gethostname")
    @patch("circus.plugins.statsd.StatsdClient")
    def setUp(self, statsd_client_mock, gethostname_mock):
        gethostname_mock.return_value = "somehost"
        envs = {
            "TSURU_APPNAME": "appname",
        }
        os.environ.update(envs)
        self.statsd = StatsdBackend()
        self.statsd.gauge = Mock()
        statsd_client_mock.assert_called_with(
            host='localhost', sample_rate=1.0, prefix='tsuru.appname.somehost', port=8125)

    def test_mem_max(self):
        self.statsd.mem_max("name", 123)
        self.statsd.gauge.assert_called_with('name.mem_max', 123)

    def test_cpu_max(self):
        self.statsd.cpu_max("name", 123)
        self.statsd.gauge.assert_called_with('name.cpu_max', 123)

    def test_mem_sum(self):
        self.statsd.mem_sum("name", 123)
        self.statsd.gauge.assert_called_with('name.mem_sum', 123)

    def test_cpu_sum(self):
        self.statsd.cpu_sum("name", 123)
        self.statsd.gauge.assert_called_with('name.cpu_sum', 123)

    def test_mem_pct_sum(self):
        self.statsd.mem_pct_sum("name", 123)
        self.statsd.gauge.assert_called_with('name.mem_pct_sum', 123)

    def test_mem_pct_max(self):
        self.statsd.mem_pct_max("name", 123)
        self.statsd.gauge.assert_called_with('name.mem_pct_max', 123)

    def test_disk_usage(self):
        self.statsd.disk_usage(123)
        self.statsd.gauge.assert_called_with('disk_usage', 123)

    def test_net_sent(self):
        self.statsd.net_sent(123)
        self.statsd.gauge.assert_called_with('net.sent', 123)

    def test_net_recv(self):
        self.statsd.net_recv(123)
        self.statsd.gauge.assert_called_with('net.recv', 123)

    def test_net_connections(self):
        self.statsd.net_connections(123)
        self.statsd.gauge.assert_called_with('net.connections', 123)

    @patch("socket.gethostname")
    def test_gauge(self, gethostname_mock):
        gethostname_mock.return_value = "somehost"
        envs = {
            "TSURU_APPNAME": "appname",
        }
        os.environ.update(envs)
        statsd = StatsdBackend()
        statsd.client = Mock()

        statsd.gauge("key", "value")

        statsd.client.gauge.assert_called_with("key", "value")
